@@ -1,10 +1,10 @@
 #!/bin/bash
-# $Id: bootstrap.sh 98 2004-07-20 01:07:06Z lennart $
+# $Id: bootstrap.sh 302 2004-11-21 17:02:25Z lennart $
 
 # This file is part of polypaudio.
 #
 # polypaudio is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
+# under the terms of the GNU Lesser General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
@@ -13,7 +13,7 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Lesser General Public License
 # along with polypaudio; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 
@@ -36,7 +36,7 @@ else
     rm -f config.cache
 
     run_versioned aclocal 1.7
-    libtoolize -c --force
+    libtoolize -c --force --ltdl
     autoheader
     run_versioned automake 1.7 -a -c --foreign
     autoconf -Wall
