@@ -1,10 +1,10 @@
-/* $Id: pactl.c 222 2004-09-19 23:12:41Z lennart $ */
+/* $Id: pactl.c 304 2004-11-21 18:15:33Z lennart $ */
 
 /***
   This file is part of polypaudio.
  
   polypaudio is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
  
@@ -13,7 +13,7 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
  
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Lesser General Public License
   along with polypaudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.
@@ -41,6 +41,10 @@
 #include <polyp/mainloop-signal.h>
 #include <polyp/sample.h>
 
+#if PA_API_VERSION != 7
+#error Invalid Polypaudio API version
+#endif
+
 #define BUFSIZE 1024
 
 static struct pa_context *context = NULL;
@@ -320,7 +324,8 @@ static void get_sink_input_info_callback(struct pa_context *c, const struct pa_s
            "Sample Specification: %s\n"
            "Volume: 0x%03x (%0.2f dB)\n"
            "Buffer Latency: %0.0f usec\n"
-           "Sink Latency: %0.0f usec\n",
+           "Sink Latency: %0.0f usec\n"
+           "Resample method: %s\n",
            i->index,
            i->name,
            i->owner_module != PA_INVALID_INDEX ? t : "n/a",
@@ -329,7 +334,8 @@ static void get_sink_input_info_callback(struct pa_context *c, const struct pa_s
            s,
            i->volume, pa_volume_to_dB(i->volume),
            (double) i->buffer_usec,
-           (double) i->sink_usec);
+           (double) i->sink_usec,
+           i->resample_method ? i->resample_method : "n/a");
 }
 
 static void get_source_output_info_callback(struct pa_context *c, const struct pa_source_output_info *i, int is_last, void *userdata) {
@@ -363,7 +369,8 @@ static void get_source_output_info_callback(struct pa_context *c, const struct p
            "Source: %u\n"
            "Sample Specification: %s\n"
            "Buffer Latency: %0.0f usec\n"
-           "Source Latency: %0.0f usec\n",
+           "Source Latency: %0.0f usec\n"
+           "Resample method: %s\n",
            i->index,
            i->name,
            i->owner_module != PA_INVALID_INDEX ? t : "n/a",
@@ -371,7 +378,8 @@ static void get_source_output_info_callback(struct pa_context *c, const struct p
            i->source,
            s,
            (double) i->buffer_usec,
-           (double) i->source_usec);
+           (double) i->source_usec,
+           i->resample_method ? i->resample_method : "n/a");
 }
 
 static void get_sample_info_callback(struct pa_context *c, const struct pa_sample_info *i, int is_last, void *userdata) {
@@ -433,11 +441,12 @@ static void get_autoload_info_callback(struct pa_context *c, const struct pa_aut
         printf("\n");
     nl = 1;
 
-    printf("*** Autoload Entry ***\n"
+    printf("*** Autoload Entry #%u ***\n"
            "Name: %s\n"
            "Type: %s\n"
            "Module: %s\n"
            "Argument: %s\n",
+           i->index,
            i->name,
            i->type == PA_AUTOLOAD_SINK ? "sink" : "source",
            i->module,
