@@ -1,13 +1,13 @@
 #ifndef foopolyplibversionhfoo /*-*-C-*-*/
 #define foopolyplibversionhfoo
 
-/* $Id: polyplib-version.h.in 139 2004-08-18 01:00:18Z lennart $ */
+/* $Id: polyplib-version.h.in 284 2004-11-14 14:58:54Z lennart $ */
 
 /***
   This file is part of polypaudio.
  
   polypaudio is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
  
@@ -16,18 +16,32 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
  
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Lesser General Public License
   along with polypaudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.
 ***/
 
+/* WARNING: Make sure to edit the real source file polyplib-version.h.in! */
+
 /** \file
  * Define header version */
 
+PA_C_DECL_BEGIN
+
 /** Return the version of the header files. Keep in mind that this is
 a macro and not a function, so it is impossible to get the pointer of
 it. */
-#define pa_get_headers_version() ("0.5.1")
+#define pa_get_headers_version() ("0.7")
+
+/** Return the version of the library the current application is linked to. */
+const char* pa_get_library_version(void);
+
+/** The current API version. Version 6 relates to polypaudio
+ * 0.6. Prior versions (i.e. Polypaudio 0.5.1 and older) have
+ * PA_API_VERSION undefined.  */
+#define PA_API_VERSION 7
+
+PA_C_DECL_END
 
 #endif
