@@ -1,13 +1,13 @@
 #ifndef foosconv_s16lefoo
 #define foosconv_s16lefoo
 
-/* $Id: sconv-s16le.h 90 2004-07-17 14:12:30Z lennart $ */
+/* $Id: sconv-s16le.h 294 2004-11-20 16:23:53Z lennart $ */
 
 /***
   This file is part of polypaudio.
  
   polypaudio is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
  
@@ -16,13 +16,13 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
  
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Lesser General Public License
   along with polypaudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.
 ***/
 
-void pa_sconv_s16le_to_float32(unsigned n, const void *a, unsigned an, float *b);
-void pa_sconv_s16le_from_float32(unsigned n, const float *a, void *b, unsigned bn);
+void pa_sconv_s16le_to_float32ne(unsigned n, const void *a, unsigned an, float *b);
+void pa_sconv_s16le_from_float32ne(unsigned n, const float *a, void *b, unsigned bn);
 
 #endif
