@@ -1,13 +1,13 @@
 #ifndef foosocketutilhfoo
 #define foosocketutilhfoo
 
-/* $Id: socket-util.h 90 2004-07-17 14:12:30Z lennart $ */
+/* $Id: socket-util.h 286 2004-11-17 00:05:25Z lennart $ */
 
 /***
   This file is part of polypaudio.
  
   polypaudio is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
  
@@ -16,7 +16,7 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
  
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Lesser General Public License
   along with polypaudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.
@@ -35,7 +35,4 @@ int pa_socket_set_rcvbuf(int fd, size_t l);
 int pa_unix_socket_is_stale(const char *fn);
 int pa_unix_socket_remove_stale(const char *fn);
 
-int pa_unix_socket_make_secure_dir(const char *fn);
-int pa_unix_socket_remove_secure_dir(const char *fn);
-
 #endif
